package com.supwisdom.dlpay

import com.supwisdom.dlpay.framework.core.JwtConfig
import com.supwisdom.dlpay.framework.core.JwtTokenUtil
import com.supwisdom.dlpay.framework.filter.ValidateCodeFilter
import com.supwisdom.dlpay.framework.security.MyInvalidSessionStrategy
import com.supwisdom.dlpay.framework.security.ValidateCodeSecurityConfig
import com.supwisdom.dlpay.framework.service.OperatorDetailService
import org.jose4j.jwt.consumer.InvalidJwtException
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.core.annotation.Order
import org.springframework.http.HttpStatus
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken
import org.springframework.security.config.annotation.web.builders.HttpSecurity
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter
import org.springframework.security.config.http.SessionCreationPolicy
import org.springframework.security.core.authority.SimpleGrantedAuthority
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.security.core.session.SessionRegistry
import org.springframework.security.core.session.SessionRegistryImpl
import org.springframework.security.core.userdetails.User
import org.springframework.security.core.userdetails.UserDetailsService
import org.springframework.security.provisioning.InMemoryUserDetailsManager
import org.springframework.security.web.authentication.AuthenticationFailureHandler
import org.springframework.security.web.authentication.AuthenticationSuccessHandler
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter
import org.springframework.security.web.session.InvalidSessionStrategy
import org.springframework.security.web.session.SessionInformationExpiredStrategy
import org.springframework.security.web.util.matcher.AntPathRequestMatcher
import org.springframework.web.filter.OncePerRequestFilter
import javax.servlet.FilterChain
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse


class ApiJwtAuthenticationFilter(jwt: JwtTokenUtil) : OncePerRequestFilter() {
    private val jwtUtil = jwt

    override fun doFilterInternal(request: HttpServletRequest, response: HttpServletResponse, filterChain: FilterChain) {
        request.getHeader(jwtUtil.header)?.let { jwt ->
            try {
                val claims = jwtUtil.verifyToken(jwt)
                val auth = UsernamePasswordAuthenticationToken(claims["uid"], null,
                        claims["authorities"]?.map { SimpleGrantedAuthority(it as String) })
                SecurityContextHolder.getContext().authentication = auth
            } catch (e: InvalidJwtException) {
                SecurityContextHolder.clearContext();
                response.sendError(HttpStatus.BAD_REQUEST.value(), e.message)
            }
        }
        filterChain.doFilter(request, response)
    }
}

@EnableWebSecurity
class WebSecurityConfig {

//    @Bean
//    fun daoProvider(detailsService: UserDetailsService): DaoAuthenticationProvider {
//        return DaoAuthenticationProvider().also {
//            it.setUserDetailsService(detailsService)
//        }
//    }
//
//    @Bean
//    fun providerManager(daoProvider: DaoAuthenticationProvider): ProviderManager {
//        return ProviderManager(listOf(daoProvider))
//    }


    companion object {
        @Configuration
        @Order(1)
        class ApiWebSecurityConfigurationAdapter : WebSecurityConfigurerAdapter() {

            @Autowired
            lateinit var jwtConfig: JwtConfig

            override fun configure(http: HttpSecurity) {
                // 设置 API 访问权限管理
                http.csrf().disable()
                        .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                        .and()
                        .addFilterAfter(ApiJwtAuthenticationFilter(JwtTokenUtil(jwtConfig)),
                                UsernamePasswordAuthenticationFilter::class.java)
                        .authorizeRequests()
                        .antMatchers("/api/auth/**").permitAll()
                        .antMatchers("/api/common/**").hasAnyRole("THIRD_COMMON", "THIRD_ADMIN")
                        .antMatchers("/api/consume/**").hasRole("THIRD_CONSUME")
                        .antMatchers("/api/deposit/**").hasRole("THIRD_DEPOSIT")
                        .antMatchers("/api/user/**").hasAnyRole("THIRD_COMMON", "THIRD_ADMIN")
                        .antMatchers("/api/shop/**").hasRole("THIRD_SHOP")
                        .anyRequest().authenticated()
                // 注册 filter
            }
        }

        @Configuration
        class MvcWebSecurityConfigurationAdapter : WebSecurityConfigurerAdapter() {
            @Autowired
            lateinit var validateCodeSecurityConfig: ValidateCodeSecurityConfig
            @Autowired
            lateinit var userDetailsService: OperatorDetailService
            @Autowired
            lateinit var zyAuthenticationFailureHandler: AuthenticationFailureHandler
            @Autowired
            lateinit var zyAuthenticationSuccessHandler: AuthenticationSuccessHandler
            @Autowired
            lateinit var myInvalidSessionStrategy: InvalidSessionStrategy


            override fun configure(http: HttpSecurity) {
                // 设置 Web MVC 应用权限
                http.apply(validateCodeSecurityConfig)
                        .and()
                        .formLogin()
                        .loginPage("/login")
                        .loginProcessingUrl("/login/form")
                        .successHandler(zyAuthenticationSuccessHandler)
                        .failureHandler(zyAuthenticationFailureHandler)
                        .and()
                        .logout()
                        .logoutRequestMatcher(AntPathRequestMatcher("/logout"))
                        .logoutSuccessUrl("/login")
                        .deleteCookies("JSESSIONID")
                        .invalidateHttpSession(true)
                        .and()
                        .userDetailsService(userDetailsService)
                        .authorizeRequests()
                        .antMatchers("/login").permitAll()
                        .antMatchers("/static/**").permitAll()
                        .antMatchers("/code/image").permitAll()
                        .anyRequest().authenticated()
                        .and()
                        .sessionManagement()
                        .invalidSessionStrategy(myInvalidSessionStrategy)
                        .maximumSessions(1)
                        .sessionRegistry(SessionRegistryImpl())
                        .maxSessionsPreventsLogin(true)
                        .and()
                        .and()
                        .headers().frameOptions().disable()
                        .and()
                        .csrf().disable()
            }
        }
    }
}
